@@ -1,13 +1,11 @@
 from datetime import datetime
-from typing import List, Optional
+from typing import List, Literal, Optional
 
-from pydantic import BaseModel, root_validator
+from pydantic import BaseModel, model_validator
 from pydantic.fields import Field
 from pydantic.networks import IPvAnyNetwork
 from pydantic.types import NonNegativeFloat
 
-from aws_lambda_powertools.utilities.parser.types import Literal
-
 from .event_bridge import EventBridgeModel
 
 
@@ -61,7 +59,7 @@ class S3Message(BaseModel):
 class S3EventNotificationObjectModel(BaseModel):
     key: str
     size: Optional[NonNegativeFloat] = None
-    etag: str
+    etag: str = Field(default="")
     version_id: str = Field(None, alias="version-id")
     sequencer: Optional[str] = None
 
@@ -101,13 +99,14 @@ class S3RecordModel(BaseModel):
     s3: S3Message
     glacierEventData: Optional[S3EventRecordGlacierEventData] = None
 
-    @root_validator(allow_reuse=True, skip_on_failure=True)
+    @model_validator(mode="before")
     def validate_s3_object(cls, values):
         event_name = values.get("eventName")
-        s3_object = values.get("s3").object
-        if "ObjectRemoved" not in event_name:
-            if s3_object.size is None or s3_object.eTag is None:
-                raise ValueError("S3Object.size and S3Object.eTag are required for non-ObjectRemoved events")
+        s3_object = values.get("s3").get("object")
+        if ":Delete" not in event_name and (s3_object.get("size") is None or s3_object.get("eTag") is None):
+            raise ValueError(
+                "Size and eTag fields are required for all events except ObjectRemoved:* and LifecycleExpiration:*.",
+            )
         return values
 
 
